
import hashlib
import secrets
import string

from pgpy import PGPKey


def _zrtp_base32(sha1: bytes) -> str:
    # https://datatracker.ietf.org/doc/html/rfc6189#section-5.1.6
    alphabet = 'ybndrfg8ejkmcpqxot1uwisza345h769'
    encoded = ''
    bits = int.from_bytes(sha1, 'big')
    shift = 155
    for j in range(32):
        n = (bits >> shift) & 31
        encoded += alphabet[n]
        shift -= 5
    return encoded


def hash_user_id(uid: str) -> str:
    if '@' in uid:
        uid, _ = uid.split('@', 1)
    # Only ASCII must be lowercased...
    lower = ''.join(c.lower() if c in string.ascii_uppercase else c for c in uid)
    digest = hashlib.sha1(lower.encode()).digest()
    return _zrtp_base32(digest)


def create_nonce(n: int = 32) -> str:
    alphabet = string.ascii_letters + string.digits
    nonce = ''.join(secrets.choice(alphabet) for _ in range(n))
    return nonce


def fingerprint(key: PGPKey) -> str:
    return key.fingerprint.upper().replace(' ', '')
